unit Decode;

interface

uses
  SysUtils,
  Classes,
  Windows,
  Inifiles,
  ZLIBArchive2Light,
  ZLIBArchiveGlobals2,
  ZLibEx,
  StrUtils,
  Encode,
  OutputStreams,
  OOCodeBaseX;

function Internal_ExtractFileToDisk(ScriptFile, InternalGroup,
  EncodedFileName, PathToExtract: string; UseTemporaryFile : Boolean = true): boolean;

function Internal_ExtractFileToDiskFromIni(IniFile: TMemIniFile;
  InternalGroup, EncodedFileName, PathToExtract: string;
  UseTemporaryFile : Boolean = true): boolean;

implementation

{$IFDEF PerformanceCounter}
uses PerformanceCounter;
{$ENDIF}

function GetTempFileName : String;
Var
  Buf1,Buf2 : ARRAY[0..MAX_PATH] Of AnsiChar;
begin
  Windows.GetTempPath(sizeOf(Buf1),@Buf1);
  Windows.GetTempFileName(@Buf2,'arc',0,@Buf2);
  result:=StrPas(Buf2);
end;

function Internal_ExtractFileToDisk(ScriptFile, InternalGroup,
  EncodedFileName, PathToExtract: string; UseTemporaryFile : Boolean = true): boolean;
var
  IniFile: TmemIniFile;
begin
  Result := False;
  {$IFDEF PerformanceCounter}
    PC.StartMeasure('Loading ini to memory');
  {$ENDIF}
  if (not FileExists(scriptFile)) then
    exit;
  IniFile := TMemIniFile.Create(ScriptFile);
  try
    Result := Internal_ExtractFileToDiskfromIni(IniFile, InternalGroup,
      EncodedFileName, PathToExtract, UseTemporaryFile);
  finally
    IniFile.Free;
  end;
end;

function Internal_ExtractFileToDiskFromIni(IniFile: TMemIniFile;
  InternalGroup, EncodedFileName, PathToExtract: string;
  UseTemporaryFile : Boolean = true): boolean;
var
  SectionName: string;
  i:     integer;
  ArcTempFile: TStream;
  ArcTempFileName: string;
  Base64Decoder: TBase64Decoder;
begin
  {$IFDEF PerformanceCounter}
  PC.StartMeasure('Initialization');
  {$ENDIF}
  Result      := False;
  ArcTempFile := nil;

  if (not Assigned(IniFile)) or (not DirectoryExists(PathToExtract)) then
    exit;

  ArcTempFileName := GetTempFileName;
  try
    If UseTemporaryFile Then
      ArcTempFile := TWin32FileStream.Create(ArcTempFileName,
        GENERIC_WRITE or GENERIC_READ, 0, CREATE_ALWAYS,
        FILE_ATTRIBUTE_TEMPORARY or FILE_FLAG_DELETE_ON_CLOSE)
    else
      ArcTempFile := TMemoryStream.Create;
    Base64Decoder := TBase64Decoder.Create(ArcTempFile);

  {$IFDEF PerformanceCounter}
    PC.StartMeasure('Searching for file-section');
  {$ENDIF}
    SectionName := 'EncodedFile-' + InternalGroup + '-' + EncodedFileName;

    if (not IniFile.SectionExists(SectionName)) then
      exit;

  {$IFDEF PerformanceCounter}
    PC.StartMeasure('Decoding lines from INI (base64)');
  {$ENDIF}
    for i := 0 to IniFile.ReadInteger(SectionName, 'lines', 0) do
      Base64Decoder.Process(IniFile.ReadString(SectionName, IntToStr(i), ''));

    FreeAndNil(Base64Decoder);

  {$IFDEF PerformanceCounter}
    PC.StartMeasure('Extracting file from archive');
  {$ENDIF}

    ExtractAll(ArcTempFile,PathToExtract);
  finally
    if Assigned(ArcTempFile) then
      ArcTempFile.Free;
    if Assigned(Base64Decoder) then
      Base64Decoder.Free;
  end;
  {$IFDEF PerformanceCounter}
  PC.StartMeasure('Finalizing');
  {$ENDIF}

  EncodedFileName := AnsiReplaceText(PathToExtract + '\' + EncodedFileName, '\\', '\');
  Result := FileExists(EncodedFileName);

  {$IFDEF PerformanceCounter}
  PC.StopMeasure;
  {$ENDIF}
end;

end.
