unit ZLibArchive2Light;

interface

uses
  Windows, SysUtils, Classes, ZLibArchiveGlobals2, ZLibEx;

procedure ExtractAll(InFile: TStream; Destination: string);

implementation

function Extract(data: TZLBFileRec2; OutFile: TStream; InStrm: TStream = nil): boolean;
var
  successfull: boolean;
  ZStream: TCustomZStream;
  fsize, count, buffersize: longint;
  buffer: pointer;

begin
  successfull := false;
  try
    InStrm.Seek(data.start, soFromBeginning);
    if (data.Status in [fsCompressed, fsCompressedEncrypted]) then
    begin
      ZStream := TZDecompressionStream.Create(InStrm);
      try
        fsize := data.osize;
        while True do
        begin
          if fsize <= 0 then
            break;
          if fsize >= 4096 then
            buffersize := 4096
          else
            buffersize := fsize;
          getmem(buffer, buffersize);
          try
            try
              Count := ZStream.Read(Buffer^, BufferSize);
              if Count <> 0 then
                OutFile.WriteBuffer(Buffer^, Count);
            except
              raise EFileCorruption.create(ArchiveCorrupted);
            end;
          finally
            freemem(buffer, buffersize);
          end;
          dec(fsize, buffersize);
        end;
      finally
        ZStream.Free;
      end;
   end
   else
   begin
      if data.osize > 0 then
         OutFile.CopyFrom(InStrm, data.osize);
   end;

   if (GetCRC32(OutFile, 0, 0) <> data.CRC) then
     raise EFileCorruption.create(Format(FileCRC, [data.name]));

    successfull := true;
  finally
    result := successfull;
  end;
end;

procedure ExtractAll(InFile: TStream; Destination: string);
var
  OutFile: TStream;
  data: TZLBFileRec2;
  extractionpath: string;
  fArchiveDetails: TZLBArchiveRec2;
  ZStream: TCustomZStream;
begin
  InFile.Seek(-sizeof(TZLBArchiveRec2), soFromEnd);
  InFile.read(fArchiveDetails, sizeof(TZLBArchiveRec2));
  if fArchiveDetails.Version = 1 then
    raise Exception.Create('V1 archives are not supported');
  if fArchiveDetails.FileCount>1 then
    raise Exception.Create('Only one file per archive supported');

  InFile.Seek(fArchiveDetails.ArchiveSize, soFromBeginning);
  Data.start:=-1;
  ZStream := TZDecompressionStream.Create(InFile);
  try
    ZStream.read(Data, sizeof(TZLBFileRec2));
  finally
    ZStream.Free;
  end;
  If (data.start=-1) Then raise
    Exception.Create('Error reading archive toc'); 

  extractionpath := destination;

  while pos('\\', extractionpath) > 0 do
    delete(extractionpath, pos('\\', extractionpath), 1);

  if extractionpath[length(extractionpath)] = '\' then
    delete(extractionpath, length(extractionpath), 1);
  ForceDirectories(extractionpath);

  if not DirectoryExists(extractionpath) then
    raise EExtractionError.create(format(FilePathError, [extractionpath]));

  OutFile := TFileStream.Create(extractionpath + '\' + data.name, fmCreate);
  try
    Extract(data, OutFile, InFile);
  finally
    OutFile.Free;
  end;
end;

end.
